/****************************************************************************
*
*	  Header Filename:  Chemistry.h
*	  Module Name:      Chemistry
*	  Description:      Defines the network of possible interactions
*                     between Molecule_Types
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2005, 2006, 2008 Chris Gordon-Smith
*     http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Chemistry_H
#define Chemistry_H


class   Molecule_Type;
class   Interaction_Type;
class   Indexed_Series;

class Chemistry

//  Represents the laws of chemistry, and holds references to
//  Molecule_Types and Interaction_Types that define the chemistry

{
    friend class Construction;
    friend class Fission;
    friend class Recombination;
    friend class Transformation;
    friend class Catalytic_Construction;
    friend class Catalytic_Fission;
    friend class Catalytic_Recombination;
    friend class Catalytic_Transformation;

    private:

//  Private Data

//===== Relationships with other classes ===================================================================

        VCSList*  TList__Of_Interaction_Types_Ptr;   // List of Interaction_Types that can
                                                     // occur within the laws of this Chhemistry
        VCSList*  TList__Of_Molecule_Types_Ptr;      // List of Molecule_Types that
                                                     // can exist in this Chemistry
//==========================================================================================================

//      Class Data

        static  Chemistry*  Chemistry__Ptr;  // Pointer to the Chemistry Object

        int Number_Of_Molecule_Types;             
        int Iter_Molecule_Type;                  // Iterator for Molecule_Types
        int Iter_Interaction_Types;              // Iterator for Interaction_Types

     public:  
	enum Rate_Const_Model_Tp {None, Arrenhius, Random};  // Needed for publc API
     private:
	Rate_Const_Model_Tp  Rate_Const_Model;   // The model to be used for determing Rate Constants
        
//  Private Functions


//  Public Functions

    public:


        void    Add_Mol_Type(Molecule_Type*  Mol_Type_Ptr);
        void    Add_Interaction_Type(Interaction_Type*  Int_Type_Ptr);
        Chemistry();
        void    Clear();
        void    Collect_Statistics(int Time);
        void    Remove_Interaction_Type(Interaction_Type*  Int_Type_Ptr);
        static  Chemistry*   Get_Chemistry__Ptr();
        Interaction_Type*  Get_Current_Int_Type_Ptr() const;
        Molecule_Type*  Get_Current_Mol_Type_Ptr() const;
        static Molecule_Type*  Get_MolType_Pt(const int   MolType_Index);
        static int  Get_Num_MolTypes();
        int     Get_Number_Of_Interaction_Types();
        int     Get_Number_Of_Constructions();
        int     Get_Number_Of_Fissions();
        void    Initialise();
        static void  Make();
        void    Set_Number_Of_Molecule_Types(int Num_Mol_Types);
        Interaction_Type*   Get_Random_IntType_Ptr();
        static  Rate_Const_Model_Tp  Get_Rate_Constant_Model();
	    static  bool    Set_Rate_Contant_Model(Rate_Const_Model_Tp  Model);

        bool    Iter_To_First_Int_Type();
        bool    Iter_To_Next_Int_Type();

        bool    Iter_To_First_Mol_Type();
        bool    Iter_To_Next_Mol_Type();

    private:

        Indexed_Series*  Num_Constructions_Series_Ptr;
        Indexed_Series*  Num_Fissions_Series_Ptr;
        Indexed_Series*  Num_IntTypes_Series_Ptr;
};

#endif  //Chemistry_H
