/****************************************************************************
*
*     Header Filename:  Krnl_Ctrl.cpp
*     Module Name:      Kernel Control API
*     Description:      API providing access to control the Kernel
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/
#include  "globldef.h"
#include  "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

//--------------  Kernel_API -------------------------------------------------------------------------------------
#include  "Krnl_Ctrl.h"
//----------------------------------------------------------------------------------------------------------------

//------------------  Kernel Manager Subsystem -------------------------------------------------------------------
#include "Simulation_Scheduler.h"
//----------------------------------------------------------------------------------------------------------------

//------------------  Simulation Subsystem -----------------------------------------------------------------------
#include "Scenario_Manager.h"
#include "Scenario_Series.h"
#include "Scenario.h"
//----------------------------------------------------------------------------------------------------------------

//------------------- Chemistry  Subsystem  ----------------------------------------------------------------------
#include "Chemistry.h"
//----------------------------------------------------------------------------------------------------------------

//------------------- Reactor    Subsystem  ----------------------------------------------------------------------
#include "Reactor.h"
//----------------------------------------------------------------------------------------------------------------

//-------------------Tracker Subsystem ---------------------------------------------------------------------------
#include "Realised_Cycle_Type_Manager.h"
#include "Tracker_Manager.h"
//----------------------------------------------------------------------------------------------------------------


Simulation_Scheduler*   Krnl_Ctrl::Sim_Scheduler_Pt = 0;
Scenario_Manager*       Krnl_Ctrl::Scenario_Mgr_Pt = 0;
Chemistry*              Krnl_Ctrl::Chem_Pt = 0;
Reactor*                Krnl_Ctrl::Reactor_Pt = 0;
Realised_Cycle_Type_Manager*  Krnl_Ctrl::RCT_Mgr_Pt = 0;
Tracker_Manager*        Krnl_Ctrl::Trckr_Mgr_Pt = 0;
//----------------------------------------------------------------------------------------------------------------

void  Krnl_Ctrl::Close_Scenario()

//  Close the current Scenario, including clearing the Chemistry and Reactor

{
    TRACE("Krnl_Ctrl::Close_Scenario");
    Scenario_Mgr_Pt->Get_Curr_Scenario_Series_Ptr()
            ->Get_Curr_Scenario_Ptr()->Clear();

    Reactor_Pt->Clear();

    Chem_Pt->Clear();

    RCT_Mgr_Pt->Clear();

    Trckr_Mgr_Pt->Clear();
}
//----------------------------------------------------------------------------------------------------------------

void  Krnl_Ctrl::Disable_Simulation_Processing()

//  Disable Simulation processing in the Kernel

{
    Sim_Scheduler_Pt->Disable_Simulation_Processing();
}
//----------------------------------------------------------------------------------------------------------------

void  Krnl_Ctrl::Enable_Simulation_Processing()

//  Ensable Simulation processing in the Kernel

{
    Sim_Scheduler_Pt->Enable_Simulation_Processing();
}
//----------------------------------------------------------------------------------------------------------------

void  Krnl_Ctrl::Initialise_API()

//  Initialise the pointers to the singleton object referenced by the API

{
    Assert(Sim_Scheduler_Pt == 0,"Krnl_Ctrl::Initialise_API - Sim_Scheduler_Ptr has already been Initialised");
    Sim_Scheduler_Pt = Simulation_Scheduler::Get_Ptr();

    Assert(Scenario_Mgr_Pt == 0,"Krnl_Ctrl::Initialise_API - Scenario_Mgr_Pt has already been Initialised");
    Scenario_Mgr_Pt = Scenario_Manager::Get_Ptr();

    Assert(Chem_Pt == 0,"Krnl_Ctrl::Initialise_API - Chem_Pt has already been Initialised");
    Chem_Pt = Chemistry::Get_Chemistry__Ptr();

    Assert(Reactor_Pt == 0,"Krnl_Ctrl::Initialise_API - Reactor_Pt has already been Initialised");
    Reactor_Pt = Reactor::Get_Reactor__Ptr();

    Assert(RCT_Mgr_Pt == 0,"Krnl_Ctrl::Initialise_API - RCT_Ngr_Pt has already been Initialised");
    RCT_Mgr_Pt = Realised_Cycle_Type_Manager::Get_Pt();

    Assert (Trckr_Mgr_Pt == 0, "Krnl_Ctrl::Initialise_API - Trckr_Mgr_Pt has already been Initialised");
    Trckr_Mgr_Pt = Tracker_Manager::Get_Pt();
}
//----------------------------------------------------------------------------------------------------------------

bool  Krnl_Ctrl::Load_Scenario_Series(const string& PathName, string&  Error_Tx)

//  Load a new Scenario Series from file 

{
    return Scenario_Mgr_Pt->Get_Curr_Scenario_Series_Ptr()
                ->Setup_From_Input_File(PathName, Error_Tx);
}
