/****************************************************************************
*
*	  Module Filename:  Scenario_Manager.cpp
*	  Module Name:      Scenario_Manager
*	  Description:      Manager for SimSoup Scenarios
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/
//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include "globldef.h"
#include "Library_Headers.h"


#include "globldef.h"
#include "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

//------------------  User Interface Manager --------------------------------------------------------------------
#include  "Simulation_Display_API.h"  //  Use only the API (keeps the subsystem dependencies simple)
//---------------------------------------------------------------------------------------------------------------

//------------------  Simulation Subsystem -----------------------------------------------------------------------
#include "Scenario_Manager.h"
#include "Scenario.h"
#include "Action_Request.h"
#include "Scenario_Series.h"
//---------------------------------------------------------------------------------------------------------------

//------------------  Reactor Subsystem -------------------------------------------------------------------------
#include "Reactor.h"
#include "Molecule_Type.h"
#include "Interaction_Type.h"
//---------------------------------------------------------------------------------------------------------------

//------------------  Chemistry Subsystem -----------------------------------------------------------------------
#include "Chemistry.h"
//---------------------------------------------------------------------------------------------------------------


Scenario_Manager*  Scenario_Manager::Instance_Ptr = 0;
//-----------------------------------------------------------------------------------------------------------------

Scenario_Series* Scenario_Manager::Get_Curr_Scenario_Series_Ptr()

//   Return pointer to current scenario series

{
    return Curr_Scenario_Series_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

Scenario_Manager*  Scenario_Manager::Get_Ptr()

//  Get a pointer to the Scenario_Manager object. It is a fatal error if the Scenario_Manager 
//  object has not yet been initialised.

{
    Assert(Instance_Ptr not_eq 0,"Scenario_Manager::Get_Ptr - Called before an instance has been created");
    return (Instance_Ptr);
}
//----------------------------------------------------------------------------------------------------------------

void  Scenario_Manager::Make()

//  Initialise the single instance Scenario_Manager object. Its a fatal error if one already exists.

{
    TRACE("Scenario_Manager::Make");
    Assert(Instance_Ptr == 0,"Scenario_Manager::Make -  Invalid call; Scenario_Manager object has already been created");    
    CREATE_OBJECT(Scenario_Manager, Instance_Ptr = new Scenario_Manager);
}
//-----------------------------------------------------------------------------------------------------------------

bool Scenario_Manager::Run_Current_Scenario_Series(uint Time_To_Run)

//  Run the current Scenario of the current Scenario Series for (about) the specified number of milliseconds
//  and return the Reactor Enabled/Disabled flag.
//  If there is no current Scenario Series then just return false to indicate Reactor must be disabled

{
    TRACE("Scenario_Manager::Run_Current_Scenario_Series");
    uint Initial_Time = Get_CPU_Time();
    if (!Curr_Scenario_Series_Ptr) return false;

    bool Reactor_Enabled = Curr_Scenario_Series_Ptr->Run_Current_Scenario(Num_Timesteps);

//  Adjust the Number of Timesteps so that we call Run_Current_Scenario about twice
//  This minimises the overhead associated with the call

    if ((Get_CPU_Time() - Initial_Time) < Time_To_Run/2)
    THEN
        Num_Timesteps = Num_Timesteps * 2;
    ELSEIF (Num_Timesteps > 1)
    THEN
        Num_Timesteps = Num_Timesteps / 2;
    ENDIF

    while((Get_CPU_Time() - Initial_Time)  < Time_To_Run)
    BEGIN
        Get_Curr_Scenario_Series_Ptr()->Run_Current_Scenario(Num_Timesteps);
    END    
    return Reactor_Enabled;
}
//----------------------------------------------------------------------------------------------------------------


Scenario_Manager::Scenario_Manager()

//  Constructor

{
    TRACE("Scenario_Manager::Scenario_Manager");
    CREATE_OBJECT(Scenario_Series, Curr_Scenario_Series_Ptr = new Scenario_Series);
    Num_Timesteps = 1;
}
