/****************************************************************************
*
*	  Module Filename:  Tracker.cpp
*	  Module Name:      Tracker
*	  Description:      Tracks paths between Molecule_Types taken by material 
*                       as Interactions take place in the Reactor
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include    "globldef.h"
#include    "Library_Headers.h"

//--------------  Tracker  Subsystem  -----------------------------------------------------------------------------
#include    "Tracker.h"
#include    "Cycle_Type.h"
#include    "Tracker_Manager.h"
#include    "Realised_Cycle_Type_Manager.h"
//-------------  Chemistry Subsystem  ---------------------------------------------------------------------------

#include    "Molecule_Type.h"

//--------------------------------------------------------------------------------------------------------------

long int  Tracker::Last_Tracker_Id = 0;

//--------------------------------------------------------------------------------------------------------------
void Tracker::Add_MolType_To_Path(Molecule_Type* Molecule_Type_Ptr)

//  Add the specified Molecule Type to the Tracker's Path
//  Check for a new cycle. If one is found then report it to the 
//  Cycle Manager and clear the recorded path so that we start a new path for future cycle checking

{
    if (Check_For_New_Cycle(Molecule_Type_Ptr) == true)
    THEN
        Realised_Cycle_Type_Manager* RCT_Ptr;
        RCT_Ptr = Tracker_Manager_Ptr->Get_Realised_Cycle_Type_Manager_Ptr();
        RCT_Ptr->Notify_Cycle_Occurrence(this,Molecule_Type_Ptr->Get_Identifier());
        MolType_Occurrences.clear();
        Tracker_Path.clear();
    ENDIF
    MolType_Occurrences.insert(std::make_pair(Molecule_Type_Ptr,1));
    Tracker_Path.push_back(Molecule_Type_Ptr);
}
//---------------------------------------------------------------------------------------------------------------

bool  Tracker::Check_For_New_Cycle(Molecule_Type* Molecule_Type_Ptr)

//  Check whether the Tracker's last step has returned it to a previously visited 
//  Molecule_Type on its path

{
    if (MolType_Occurrences.find(Molecule_Type_Ptr) not_eq MolType_Occurrences.end())
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//---------------------------------------------------------------------------------------------------------------

void  Tracker::Destroy(Tracker* Tracker_Ptr)

//  Destroy the specified Tracker

{
    Tracker_Ptr->Tracker_Path.clear();
    Tracker_Ptr->MolType_Occurrences.clear();
    DELETE_OBJECT(Tracker, Tracker_Ptr);
}
//---------------------------------------------------------------------------------------------------------------

Molecule_Type*  Tracker::Get_Current_MolType_Ptr()

//  Get a pointer to the Molecule Type at which the Tracker Path Iterator is currently placed

{
    return (*Tracker_Path_Iterator);
}
//---------------------------------------------------------------------------------------------------------------

void Tracker::Get_Path_As_String(string& Path_As_String)

//  Return the Tracker's path in String format

{
    Path_As_String = string("Tracker ") + To_String(Tracker_Id);
    list<Molecule_Type*>::iterator  Iter;
    for (Iter = Tracker_Path.begin();
        Iter not_eq Tracker_Path.end();
        Iter++)
    BEGIN
        Path_As_String += ":" + To_String((*Iter)->Get_Identifier());
    END
}
//---------------------------------------------------------------------------------------------------------------

int Tracker::Get_Path_Length()

{
    return Tracker_Path.size();
}
//---------------------------------------------------------------------------------------------------------------

bool Tracker::Iter_To_First_MolType()

//  Set the Tracker Path iterator to the first item

{
    if (Tracker_Path.empty())
    THEN
        return false;
    ELSE
        Tracker_Path_Iterator = Tracker_Path.begin();
        return true;
    ENDIF
}
//---------------------------------------------------------------------------------------------------------------
        
bool Tracker::Iter_To_Next_MolType()

//  Set the Tracker Path iterator to the next item

{
    if (++Tracker_Path_Iterator not_eq Tracker_Path.end())
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//--------------------------------------------------------------------------------------------------------------

void Tracker::Set_Iter_To_Place_In_List(list<Tracker*>::iterator List_Position)

//  Mark the Tracker with an iterator to its place in a list.
//  This enables fast removal from the list.

{
    Iter_To_Place_In_List = List_Position;
}
//---------------------------------------------------------------------------------------------------------------

Tracker::Tracker(Tracker_Manager* Tracker_Mgr_Ptr)

//  Constructor

{
    Last_Tracker_Id++;
    Tracker_Id = Last_Tracker_Id;
    Tracker_Manager_Ptr = Tracker_Mgr_Ptr;
} 
//----------------------------------------------------------------------------------------------------------------

Tracker::~Tracker()

{
   
}
