/****************************************************************************
*
*	  Header Filename:  Simulation_Statistics.h
*	  Module Name:      Simulation_Statistics
*	  Description:      Records statistics gathered during running of simulation
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:- 
*
*       Version    Date                        Update Description
*       =======    ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Simulation_Statistics_H
#define Simulation_Statistics_H

//---------------------------------------------------------------------------------------------------------------
// Notes on Usage:

//   Create_Indexed_Series must be called before a Series can be used
//   Select a Series with Set_Current_Indexed_Series
//   Go to the first Indexed Series using Iter_To_First_Ind_Series()
//   Go from one Indexed Series to the next with Iter_To_Next_Ind_Series()
//---------------------------------------------------------------------------------------------------------------

class   Indexed_Series;

class   Simulation_Statistics

{
//===== Relationships with other classes ===================================================================
    private:
        map  <string, pair<bool, vector<Indexed_Series*> > > List_Of_Indexed_Series_Lists;
                                                             // bool indicates whether series list is
                                                             // intended for user display
//==========================================================================================================

    public:                
//        void    Clear();           
        Indexed_Series* Create_Indexed_Series(string Indexed_Series_Name,
                                              string Index_Name, 
                                              string Value_Name,
                                              string Series_List,
                                              bool No_Entry_Means_Zero);

        void  Clear_Series_List(string List_Name);

        void  Create_And_Populate_Manhattn_Dist_Ser
                                (string Manhattan_Series_Name,      // Name of the Manhattan Series to be
                                                                    // created and populated
                                 string Manhattan_Series_List_Name, // The name of the list of Manhattan Series
                                 string Base_Data_Series_List_Name, // Name of the list of base data series from 
                                                                    // which the Manhattan series is to be produced
                                 double Last_Vector_Index,          // Index of the last base dataseries 
                                 int Vector_Index_Increment);       // Index increment between the base data series

        void  Delete_Series(Indexed_Series* Series_Pt);
        void  Create_Indexed_Series_List(string Series_List, bool Is_For_User_Access_Fl);
        Indexed_Series* Get_Current_Indexed_Series_Ptr();
        string Get_Curr_SerList_Name();
        bool   Get_Curr_SerList_Usr_Access();
        
        bool    Iter_To_First_Ind_Series();
        bool    Iter_To_Next_Ind_Series();
        bool    Iter_To_Prev_Ind_Series();
        bool    Iter_To_Last_Ind_Series();
        bool    Iter_To_First_Ind_Series_List();
        bool    Iter_To_Next_Ind_Series_List();
        void    Set_Current_Ind_Series_List(string Series_List);
        void    Set_Default_SerList(string Series_List);
        string  Get_Default_SerList_Nm();
        static  Simulation_Statistics*  Get_Ptr();
        static void  Make();
        
    private:   
        Simulation_Statistics();
        unsigned int Current_Indexed_Series_Index;
        map  <string, pair<bool, vector<Indexed_Series*> > >::iterator List_Of_Ind_Series_Lists_Iter;
        static Simulation_Statistics*  Instance_Ptr;
        string Default_SerList_Tx_;
//        string  Current_Indexed_Series_Name;
//        int    Collection_Interval;
};
#endif  //Simulation_Statistics_H
