/****************************************************************************
*
*	  Module Filename:  Form_Simulation_Control.cpp
*	  Module Name:      Form_Simulation_Control
*	  Description:      Simulation Control Panel
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

//#define  ENABLE_TRACE_FILE  

#include    "globldef.h"
#include    "Library_Headers.h"

//----------------  User_Interface_Presentation API --------------------------------------------------------------
#include  "GUI_Presentation_API.h"
//----------------------------------------------------------------------------------------------------------------

//  User Interface Maneger --------------------------------------------------------------------------------------
#include    "Form_Simulation_Control.h"
#include    "Network_Painter_Prot.h"
#include    "Form_View_Reactor.h"
#include    "Form_View_Chemistry.h"
#include    "Form_System_Monitor.h"
#include    "Form_Simulation_Statistics.h"
#include    "Simulation_Display_API.h"
//---------------------------------------------------------------------------------------------------------------

//  Statistics Subsystem -----------------------------------------------------------------------------------------

#include    "Simulation_Statistics.h"
//---------------------------------------------------------------------------------------------------------------

//------------------ Kernel Manager -----------------------------------------------------------------------------
#include "Krnl_Vis.h"
//---------------------------------------------------------------------------------------------------------------

Form_Simulation_Control* Form_Simulation_Control::Instance_Ptr = 0;

//=================================================================================================================

void  Form_Simulation_Control::Clear_Log()

//  Clear the simulation Log

{
    GUI::Clear(Widget_Ids::SimControl_Log_TextBuffer);
}
//----------------------------------------------------------------------------------------------------------------

bool  Form_Simulation_Control::Exists()

//  Return true if the singleton Simulation Cintrol Form exists

{
    if (Instance_Ptr not_eq 0)
    THEN
        return true;
    ELSE
        return false;
    ENDIF   
}
//-----------------------------------------------------------------------------------------------------------------

void  Form_Simulation_Control::Export_Scenario()

//  Request Scenario Manager to produce an sbml file defining the scenario

{
//    Scenario_Manager::Get_Ptr()->Get_Curr_Scenario_Series_Ptr()->Get_Curr_Scenario_Ptr()->Export();
}
//-----------------------------------------------------------------------------------------------------------------
Form_Simulation_Control::Form_Simulation_Control()

{  
 
}
//----------------------------------------------------------------------------------------------------------------

Form_Simulation_Control*  Form_Simulation_Control::Get_Ptr()

//  Get the pointer to the singleton Form_Simulation_Control object.
//  Its a fatal error if the object has not yet been created.

{
    Assert(Instance_Ptr not_eq 0,"Form_Simulation_Control::Get_Ptr: Called before an instance has been created");
    return (Instance_Ptr);
}
//---------------------------------------------------------------------------------------------------------------

void    Form_Simulation_Control::List_Current_Scenario()

//  List the current Scenario's Action Requests in the ListView

{
    TRACE("Form_Simulation_Control::List_Current_Scenario");
    list<string>  List_Of_Strings;

    GUI_Presentation_API::Clear(Widget_Ids::Simulation_Actions_ListStore);
               
    Scenario* Scenario_Ptr = Krnl_Vis::Scenario_::Get_Curr_Scenario_Ptr();
    if (!Scenario_Ptr ) return;  // Nothing to list if there is no scenario
    
//  Loop over Action Requests for the current Scenario

    for (bool Continue_Flag = Krnl_Vis::Scenario_::Iter_To_First_Action_Request(Scenario_Ptr);
    Continue_Flag == true;
    Continue_Flag = Krnl_Vis::Scenario_::Iter_To_Next_Action_Request(Scenario_Ptr))
    BEGIN
        Action_Request*  Curr_Act_Req_Ptr = Krnl_Vis::Scenario_::Get_Current_Action_Req_Ptr(Scenario_Ptr);
        if (not Krnl_Vis::Action_Request_::Get_Repeat_Req_Flag(Curr_Act_Req_Ptr))  // Don't list auto generated Action Requests
        THEN
//          Display the time, Type and Subject for the Action Request...

            List_Of_Strings.push_back(To_String(Krnl_Vis::Action_Request_::Get_Time(Curr_Act_Req_Ptr)));
            List_Of_Strings.push_back(To_String(Krnl_Vis::Action_Request_::Get_Type(Curr_Act_Req_Ptr) + " " 
                                                + Krnl_Vis::Action_Request_::Get_Subject(Curr_Act_Req_Ptr)));

//          ... and the first parameter if there is one

            list<string> Param_List;
            list<string>::iterator  Param_List_Iter;
            Krnl_Vis::Action_Request_::Get_Params(Curr_Act_Req_Ptr, Param_List);
            Param_List_Iter = Param_List.begin();

            if (Param_List_Iter not_eq Param_List.end())
            THEN
                List_Of_Strings.push_back(*Param_List_Iter);

                if (Krnl_Vis::Action_Request_::Get_Repeat_Interval(Curr_Act_Req_Ptr))
                THEN
                    string Repeat_Time(To_String(Krnl_Vis::Action_Request_::Get_Repeat_Interval(Curr_Act_Req_Ptr)));
                    List_Of_Strings.push_back(string(Repeat_Time));
                ELSE
                    List_Of_Strings.push_back(string("No Repeat"));
                ENDIF
                ++Param_List_Iter;
            ENDIF

            GUI_Presentation_API::Add_Row(Widget_Ids::Simulation_Actions_ListStore, 
                                          List_Of_Strings, 0); 

//          Loop over other parameters for the Action Request.  Display one per line.           
           
            while (Param_List_Iter not_eq Param_List.end())
            BEGIN
                List_Of_Strings.clear();
//                List_Of_Strings.push_front(string(""));
                List_Of_Strings.push_front(string(""));
                List_Of_Strings.push_front(string(""));
                List_Of_Strings.push_back(*Param_List_Iter);

                GUI_Presentation_API::Add_Row(Widget_Ids::Simulation_Actions_ListStore, 
                                            List_Of_Strings, 0); 
                ++Param_List_Iter;
            END  
        ENDIF                                
    END 
}
//-----------------------------------------------------------------------------------------------------------------

void  Form_Simulation_Control::Make()

//  Create the Form_Simulation_Control singleton object and initialise display

{
    Assert(Instance_Ptr == 0,"Form_Simulation_Control::Make -  Invalid call; Form_Simulation_Control has already been created");
    CREATE_OBJECT(Form_Simulation_Control, Instance_Ptr = new Form_Simulation_Control);   
}
//-----------------------------------------------------------------------------------------------------------------

//void  Form_Simulation_Control::New_Scenario(bool Warn_User)

//  Call Scenario Manager to create a new Scenario
//{
//    if (Warn_User)
//    THEN
//        if (TApplication::MessageBox(string("This will delete the current scenario and any simulation results."), string ("Notification"), smbCancel)
//                not_eq smbCancel)  // This works, even though the smbCancel logic looks wrong (would expect == smbCancel)
//        THEN
//            return;
//        ENDIF
//    ENDIF

//      Simulation_Statistics::Get_Ptr()->Clear();
//       Krnl_Vis::Reactor_::Clear();
//       Krnl_Vis::Chemistry_::Clear();
//       Krnl_Vis::Reactor_::Initialise();
//       Krnl_Vis::Chemistry_::Initialise();

//     Redisplay_Form();
// 
// //    Form_Simulation_Statistics::Get_Ptr()->Clear();
//     TForm__View_Chemistry::Get_Ptr()->Clear();
//     TForm__View_Reactor::Get_Ptr()->Clear();
//     Form_System_Monitor::Get_Ptr()->Display_Object_Statistics();
//     
//     TForm__View_Chemistry::Show_Chemistry(200);  
//     TForm__View_Reactor::Show_Reactor(200);   
//     
// //    Form_Simulation_Statistics::Get_Ptr()->Initialise();
// 
//     Output_Simple_Log_Entry(string(""));
//     Output_Simple_Log_Entry(string("SCENARIO LOG:-"));
//     Output_Simple_Log_Entry(string(""));
//} 
//---------------------------------------------------------------------------------------------------------------

void   Form_Simulation_Control::Output_Default_Parameters_To_Log()

//  Output the Simulation Parameters

{
    Output_Simple_Log_Entry(string("Default Reactor Volume is ") + To_String(Krnl_Vis::Reactor_::Get_Volume()));
    Output_Simple_Log_Entry(string("Default Maximum Molecule Mass is ") + To_String(Krnl_Vis::MolType::Get_Max_Mass()));
    Output_Simple_Log_Entry(string("Default Statistics Collection Interval is ") + To_String(Krnl_Vis::Scenario_::Get_Statistics_Collection_Interval()));
    Output_Simple_Log_Entry(string("Default Max. Rate Constant - Constructions is ") + To_String(Krnl_Vis::Construction_::Get_Max_Rand_Rate_Constant()));
    Output_Simple_Log_Entry(string("Default Max. Rate Constant - Fissions is ") + To_String(Krnl_Vis::Fission_::Get_Max_Rand_Rate_Constant()));
//    Output_Simple_Log_Entry(string("Default Max. Rate Constant - Transformation is ") + To_String(Transformation::Get_Max_Rate_Constant()));
    Output_Simple_Log_Entry(string("Outflow: Default Proportion of Molecules to Remove per Timestep is ") + To_String(Krnl_Vis::Reactor_::Get_Leakage_Proportion()));
}
//-----------------------------------------------------------------------------------------------------------------

void   Form_Simulation_Control::Output_Log_Entry(string Log_Entry)

//  Make an entry on the Simulation Log

{
    int Time = Krnl_Vis::Reactor_::Get_Time();
    string  Output_Line = string("Time = ") + To_String(Time) + string(": ") + Log_Entry;
    cout << Output_Line  << endl;
    GUI::Add_Text(Widget_Ids::SimControl_Log_TextBuffer, Output_Line);

    string NewLine("\n");
    GUI::Add_Text(Widget_Ids::SimControl_Log_TextBuffer, NewLine);
}
//---------------------------------------------------------------------------------------------------------------

void   Form_Simulation_Control::Output_Simple_Log_Entry(string Log_Entry)

//  Output a log entry to the simulation log without including the time

{
    GUI::Add_Text(Widget_Ids::SimControl_Log_TextBuffer, Log_Entry);
    cout << Log_Entry  << endl;

    string NewLine("\n");
    GUI::Add_Text(Widget_Ids::SimControl_Log_TextBuffer, NewLine);
}
//----------------------------------------------------------------------------------------------------------------

void  Form_Simulation_Control::Redisplay_Form()

//  Redisplay form (eg if it is resized)

{
    List_Current_Scenario();
}
//-----------------------------------------------------------------------------------------------------------------

