/****************************************************************************
*
*	  Header Filename:  Utility_Functions.h
*	  Module Name:      Utility_Functions
*	  Description:      General Utility Functions
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Utility_Functions_H
#define Utility_Functions_H

namespace  Utility_Functions {

//  Assert template.  Displays a message and exits if the assert condition is not true

template<class A>  inline void Assert(A assertion, const char* Error_Message__Ptr)
{
//
//  Under GCC, this only seems to link properly if VCS_Functions::Display_Exception_Message
//  is called non-inline from somewhere
//
    if(!assertion)
    THEN
        cout << "SimSoup Assertion Failure - " <<  Error_Message__Ptr  <<  endl;
        VCS_Functions::Display_Exception_Message(Error_Message__Ptr,"SimSoup Assertion Failure");
        exit(EXIT_FAILURE);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

template<class T>     // Declaration
std::string To_String (const T &x);

template<class T>    // Definition
std::string To_String (const T &x)

//  Utility function to return a string containing
//  the converted argument
//  If its floating point, output in fixed format with 8 decimal places

{
    std::ostringstream oss;
//    oss << std::fixed << std::setprecision(8) << std::noshowpoint;
    oss << x;
    return oss.str();
}

//-----------------------------------------------------------------------------------------------------------------

uint    Get_CPU_Time();
string  Get_Error_Tx();
string  Get_Warn_Tx();
string  Get_Equals_Tx();
string  Get_ErrLine_Tx();
string  String_In_Quotes(string Tx);
bool    Text_To_Float(string Text, double& Value);
bool    Text_To_Int(string& Text, int& Value);
bool    Text_To_PosInt(string& Text, int& Value);
bool    Validate_Text_As_Non_Neg_Int(string Text);
bool    Validate_Text_As_Pos_Int(string Text);
bool    Validate_Text_As_Non_Neg_Float(string Text);
string  Text_To_Standard_Number_Format(string Text);
//----------------------------------------------------------------------------------------------------------

bool    Validate_Unique_Integers(int int1, int int2);
bool    Validate_Unique_Integers(int int1, int int2, int int3);
bool    Validate_Unique_Integers(int int1, int int2, int int3, int int4);
bool    Validate_Unique_Integers(int int1, int int2, int int3, int int4, int int5);

//-----------------------------------------------------------------------------------------------------------


}    // Close Namespace

using namespace Utility_Functions;

#endif  //Utility_Functions_H     
