/****************************************************************************
*
*	  Module Filename:  VCL_Compatibility_Support.cpp
*	  Module Name:      VCL_Compatibility_Support
*	  Description:      Provides some VCL compatible features without using the VCL.
*                       This is to maximise portability.
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/
#include "globldef.h"
#include "Library_Headers.h"

#include  <gsl/gsl_rng.h>    // GNU Scientific library
#include  "Random_Sequence.h"

int VCSList::Add(void*  Item_Ptr)

//  Add an item to the VCSList

{
    try
    {
        Items.push_back(Item_Ptr);
    }

    catch(...)

    {
        Assert (true == false, "VCSList::Add - Failed to Add item");
//        TApplication::MessageBox("VCSList::Add - Failed to Add item", "Exception", smbOK);   
    }

    return (0);  
}
//----------------------------------------------------------------------------------------------------------
void  VCSList::Clear()

//  Empty the VCSList

{
    Items.clear();
}
//----------------------------------------------------------------------------------------------------------

int  VCSList::Count()

//    Return number of items in the VCSList

{
    return(Items.size());
}
//-----------------------------------------------------------------------------------------------------------------

bool  VCSList::Iter_To_First()

//  Set the Iterator to the first item if it exists

{
    if(Items.begin() == Items.end())
    THEN
        return false;
    ELSE
        Iterator = Items.begin();
        return true;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------

void*  VCSList::Get_Current_Item()

//  Return the current item

{
    return(*Iterator);
}
//-----------------------------------------------------------------------------------------------------------------
void*  VCSList::Get_Item(int Index)

//    Return the requested item

{
    return(Items[Index]);
}
//----------------------------------------------------------------------------------------------------------------

void*   VCSList::Get_Random_Value()

//  Get an item value at random from the VCS List, and return it to the caller.
//  If the same value occurs multiple times, then it is the first 
//  occurrence of the value that is removed
//  If the list is empty, then return 0.

{
    int Num_Items = Items.size();
    if (Num_Items < 1)
    THEN
        return 0;
    ELSE
        int Index_To_Get = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Num_Items);
        void* Val_To_Get = Items[Index_To_Get];
        return Val_To_Get;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------

bool  VCSList::Iter_To_Next()

//  Set the Iterator to the next item if it exists

{
    if(++Iterator == Items.end())
    THEN
        return false;
    ELSE
        return true;
    ENDIF
}
//------------------------------------------------------------------------------------------------------------------
void VCSList::Remove(void*  Item_Ptr)

//  Remove the specified item from the list
//  This function is not optimised, and so is suitable only for use when infrequently called

{
    vector<void*>::iterator  Iter;
    Iter = find(Items.begin(), Items.end(), Item_Ptr);

    Assert (Iter != Items.end(), "VCSList::Remove - Invalid argument");

    Items.erase(Iter);
}
//----------------------------------------------------------------------------------------------------------
void VCSList::Remove_Last()

//  Remove the last item from the VCSList

{
    Assert (Items.empty() not_eq true, "VCSList::Remove_Last - Attempt to remove from empty vector");
    Items.pop_back();
}
//----------------------------------------------------------------------------------------------------------------

void*    VCSList::Remove_Random_Value()

//  Remove an item value at random from the VCS List, and return it to the caller.
//  If the same value occurs multiple times, then it is the first 
//  occurrence of the value that is removed

{
    Assert (Items.empty() not_eq true, "VCSList::Remove_Random_Value - Attempt to remove from empty vector");

    int Num_Items = Items.size();
    int Index_To_Remove = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Num_Items);
    void* Val_To_Remove = Items[Index_To_Remove];
    vector<void*>::iterator  Iter;
    Iter = find(Items.begin(),Items.end(), Val_To_Remove);  // See Josuttis page 154

    Assert (Iter not_eq Items.end(), "VCSList::Remove_Random_Value - Value to remove not found");
    Items.erase(Iter);
    return Val_To_Remove;
}
//----------------------------------------------------------------------------------------------------------

void VCS_Functions::Display_Exception_Message(char const* Error_Message__Ptr,char const* Error_Type__Ptr)

//   Display an Exception with the specified message and Exception Type

{
//    TApplication::MessageBox(Error_Message__Ptr, Error_Type__Ptr, smbOK);

//    Assert(true == false, Error_Message__Ptr);
}
//----------------------------------------------------------------------------------------------------------

